@@ -29,7 +29,10 @@ namespace Rcpp {
     namespace internal {
 
         template <typename T> T primitive_as(SEXP x) {
-            if (::Rf_length(x) != 1) throw ::Rcpp::not_compatible("expecting a single value");
+            if (::Rf_length(x) != 1) {
+                const char* fmt = "Expecting a single value: [extent=%i].";
+                throw ::Rcpp::not_compatible(fmt, ::Rf_length(x));
+            }
             const int RTYPE = ::Rcpp::traits::r_sexptype_traits<T>::rtype;
             Shield<SEXP> y(r_cast<RTYPE>(x));
             typedef typename ::Rcpp::traits::storage_type<RTYPE>::type STORAGE;
@@ -43,10 +46,14 @@ namespace Rcpp {
 
         inline const char* check_single_string(SEXP x) {
             if (TYPEOF(x) == CHARSXP) return CHAR(x);
-            if (! ::Rf_isString(x))
-                throw ::Rcpp::not_compatible("expecting a string");
-            if (Rf_length(x) != 1)
-                throw ::Rcpp::not_compatible("expecting a single value");
+            if (! ::Rf_isString(x) || Rf_length(x) != 1) {
+                const char* fmt = "Expecting a single string value: "
+                                  "[type=%s; extent=%i].";
+                throw ::Rcpp::not_compatible(fmt,
+                                             Rf_type2char(TYPEOF(x)),
+                                             Rf_length(x));
+            }
+
             return CHAR(STRING_ELT(::Rcpp::r_cast<STRSXP>(x), 0));
         }
 
@@ -66,10 +73,11 @@ namespace Rcpp {
 
         template <typename T> T as(SEXP x, ::Rcpp::traits::r_type_RcppString_tag) {
             if (! ::Rf_isString(x)) {
-                throw ::Rcpp::not_compatible("expecting a string");
-            }
-            if (Rf_length(x) != 1) {
-                throw ::Rcpp::not_compatible("expecting a single value");
+                const char* fmt = "Expecting a single string value: "
+                                  "[type=%s; extent=%i].";
+                throw ::Rcpp::not_compatible(fmt,
+                                             Rf_type2char(TYPEOF(x)),
+                                             Rf_length(x));
             }
             return STRING_ELT(::Rcpp::r_cast<STRSXP>(x), 0);
         }
