@@ -40,11 +40,11 @@ public:
 	inline STORAGE operator[]( int i ) const {
 		return object( i, i ) ;
 	}
-	inline int size() const { return n; }
+	inline R_xlen_t size() const { return n; }
 
 private:
 	const MAT_TYPE& object ;
-	int n ;
+	R_xlen_t n ;
 } ;
 
 
@@ -59,7 +59,7 @@ public:
 	inline STORAGE operator()( int i, int j ) const {
 		return (i==j) ? object[i] : 0 ;
 	}
-	inline int size() const { return n * n; }
+	inline R_xlen_t size() const { return static_cast<R_xlen_t>(n) * n; }
 	inline int ncol() const { return n; }
 	inline int nrow() const { return n; }
 
